"""praevisio: a CLI tool for AI governance through verifiable promises.

This package exposes the application and domain layers used by the CLI. See the
API reference in the documentation site for details.
"""

__all__ = ["__version__"]

__version__ = "0.0.0"
